#!/usr/bin/env python
'''
D3D it with Python
-------------------------------------------------------------------------------
0.1 - Inital release
'''
__author__ = "M.J. Roy"
__version__ = "0.1"
__email__ = "matthew.roy@manchester.ac.uk"
__status__ = "Experimental"
__copyright__ = "(c) M. J. Roy, 2024-"

import os, sys
import numpy as np
import vtk
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vtk.util.numpy_support import vtk_to_numpy as v2n
from PyQt5 import QtGui, QtWidgets, QtCore
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib import rc
from matplotlib.patches import Polygon
import importlib.resources

from d3dslic3r.d3dslic3r_common import *
from d3dslic3r.d3dslic3r_gui_common import *
from d3dslic3r.export_widget import export_widget
from d3dslic3r.transform_widget import make_transformation_button_layout, get_trans_from_euler_angles


def launch(*args, **kwargs):
    '''
    Start Qt/VTK interaction if started independently
    '''
    app = QtWidgets.QApplication.instance()
    if app is None:
        app = QtWidgets.QApplication(sys.argv)

    app.processEvents()
    
    window = interactor(None) #otherwise specify parent widget
    window.show()
    
    ret = app.exec_()
    
    if sys.stdin.isatty() and not hasattr(sys, 'ps1'):
        sys.exit(ret)
    else:
        return window

class main_window(QtWidgets.QWidget):
    """
    Generic object containing all UI
    """
    
    def setup(self, MainWindow):
        '''
        Creates Qt interactor
        '''
        
        #if called as a script, then treat as a mainwindow, otherwise treat as a generic widget
        if hasattr(MainWindow,'setCentralWidget'):
            MainWindow.setCentralWidget(self.centralWidget)
        else:
            self.centralWidget=MainWindow
        MainWindow.setWindowTitle("slic3 widget v%s" %__version__)
        MainWindow.setMinimumSize(QtCore.QSize(500, 500)) #figsize(8,6)
        
        #create new layout to hold both VTK and Qt interactors
        mainlayout=QtWidgets.QHBoxLayout(self.centralWidget)

        #create VTK widget
        self.vtkWidget = QVTKRenderWindowInteractor(self.centralWidget)
        
        #create Qt layout to contain interactions
        io_layout = QtWidgets.QGridLayout()
        
        #create VTK widget
        self.vtkWidget = QVTKRenderWindowInteractor(self.centralWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(100)
        sizePolicy.setVerticalStretch(100)
        self.vtkWidget.setSizePolicy(sizePolicy)
        
        #set fonts
        head_font=QtGui.QFont("Helvetica [Cronyx]",weight=QtGui.QFont.Bold)
        io_font = QtGui.QFont("Helvetica")
        
        #make display layout
        io_box = QtWidgets.QGroupBox("I/O")
        #buttons
        self.load_button = QtWidgets.QPushButton('Load')
        self.load_label = QtWidgets.QLabel("Nothing loaded.")
        self.load_label.setWordWrap(True)
        self.load_label.setFont(io_font)
        self.load_label.setToolTip('Load STL file for slicing')
        #make combo box for slices
        self.slice_num_cb = QtWidgets.QComboBox()
        self.slice_num_cb.setToolTip('Change slice number highlighted')
        self.slice_num_cb.setEnabled(False)
        self.export_slice=QtWidgets.QPushButton("Export")
        self.export_slice.setToolTip('Export indicated slice number')
        self.export_slice.setCheckable(True)
        self.export_slice.setChecked(False)
        self.export_slice.setEnabled(False)
        
        io_layout.addWidget(self.load_button,0,0,1,1)
        io_layout.addWidget(self.slice_num_cb,0,1,1,1)
        io_layout.addWidget(self.export_slice,0,2,1,1)
        io_layout.addWidget(self.load_label,1,0,1,3)
        
        
        #Geometry manipulation layout
        geo_button_layout = QtWidgets.QGridLayout()
        self.geo_box = collapsible_box("Geometry manipulation")
        self.op_slider_label = QtWidgets.QLabel("Opacity:")
        self.op_slider = QtWidgets.QSlider(QtCore.Qt.Horizontal)
        self.op_slider.setRange(0,100)
        self.op_slider.setSliderPosition(100)
        
        self.export_stl_button = QtWidgets.QPushButton('Export')
        self.export_stl_button.setToolTip('Export current model geometry as new STL')
        
        geo_button_layout.addWidget(self.op_slider_label,0,0,1,1)
        geo_button_layout.addWidget(self.op_slider,0,1,1,1)
        geo_button_layout.addWidget(self.export_stl_button,0,2,1,1)
        geo_button_layout.addLayout(make_transformation_button_layout(self),1,0,1,2)
        geo_button_layout.setColumnStretch(0, 0)
        geo_button_layout.setColumnStretch(1, 1)
        
        self.geo_box.setEnabled(False)
        
        #make slice layout
        self.slice_box = QtWidgets.QGroupBox('Slice')
        slice_layout = QtWidgets.QGridLayout()
        self.spacing_rb=QtWidgets.QRadioButton("Spacing")
        self.quantity_rb=QtWidgets.QRadioButton("Quantity")
        self.spacing_rb.setChecked(True)
        self.slice_rb_group = QtWidgets.QButtonGroup()
        self.slice_rb_group.addButton(self.spacing_rb)
        self.slice_rb_group.addButton(self.quantity_rb)
        self.slice_rb_group.setExclusive(True)
        self.by_height_sb = QtWidgets.QDoubleSpinBox()
        self.by_height_sb.setSuffix(' mm')
        self.by_height_sb.setToolTip('Average height of each slice')
        self.by_height_sb.setMinimum(0.001)
        self.by_height_sb.setMaximum(1000)
        self.by_height_sb.setDecimals(3)
        self.by_height_sb.setValue(4)
        self.by_num_sb = QtWidgets.QSpinBox()
        self.by_num_sb.setPrefix('N = ')
        self.by_num_sb.setMinimum(4)
        self.by_num_sb.setMaximum(10000)
        self.by_num_sb.setValue(20)
        self.by_num_sb.setToolTip('Specify number of slices')
        self.agglom_param_sb = QtWidgets.QDoubleSpinBox()
        self.agglom_param_sb.setPrefix('Threshold = ')
        self.agglom_param_sb.setMinimum(0.001)
        self.agglom_param_sb.setMaximum(10000)
        self.agglom_param_sb.setDecimals(1)
        self.agglom_param_sb.setValue(5)
        self.agglom_param_sb.setToolTip('Threshold for sub-slicing')
        self.update_slice_button = QtWidgets.QPushButton('Slice')
        self.update_slice_button.setToolTip('Slice with given paramters')

        #populate slice box
        slice_layout.addWidget(self.spacing_rb, 1, 0, 1, 1)
        slice_layout.addWidget(self.quantity_rb, 1, 1, 1, 1)
        slice_layout.addWidget(self.by_height_sb,2,0,1,1)
        slice_layout.addWidget(self.by_num_sb,2,1,1,1)
        slice_layout.addWidget(self.agglom_param_sb,3,0,1,1)
        slice_layout.addWidget(self.update_slice_button,3,1,1,1)
        
        self.slice_box.setEnabled(False)
        
        
        #make pathing box
        self.path_box = QtWidgets.QGroupBox('Pathing')
        path_layout = QtWidgets.QVBoxLayout()
        
        #make path_gen layout
        path_gen_layout = QtWidgets.QGridLayout()
        self.path_all_cb = QtWidgets.QCheckBox('Path all')
        self.path_all_cb.setEnabled(False)
        self.path_all_cb.setToolTip('Apply to all slices')
        self.path_outline_cb = QtWidgets.QCheckBox('Path outline')
        self.path_outline_cb.setToolTip('Include outline in pathing')
        self.rotate_z_path_sb = QtWidgets.QDoubleSpinBox()
        self.rotate_z_path_sb.setToolTip('Rotational offset for pathing of slice. Positive is clockwise.')
        self.rotate_z_path_sb.setValue(0)
        self.rotate_z_path_sb.setMaximum(180)
        self.rotate_z_path_sb.setMinimum(-180)
        self.rotate_z_path_sb.setPrefix("\u03b8 = ")
        self.rotate_z_path_sb.setSuffix("\u00b0")
        self.by_width_sb = QtWidgets.QDoubleSpinBox()
        self.by_width_sb.setSuffix(' mm')
        self.by_width_sb.setToolTip('Average width of each path/bead')
        self.by_width_sb.setMinimum(0.001)
        self.by_width_sb.setMaximum(1000)
        self.by_width_sb.setDecimals(3)
        self.by_width_sb.setValue(4)
        self.num_paths_label = QtWidgets.QLabel("No paths.")
        self.num_paths_label.setWordWrap(True)
        self.num_paths_label.setFont(io_font)
        self.num_paths_label.setToolTip('Total number of paths/beads resolved')
        self.bead_offset_sb = QtWidgets.QDoubleSpinBox()
        self.bead_offset_sb.setMinimum(30)
        self.bead_offset_sb.setMaximum(80)
        self.bead_offset_sb.setDecimals(3)
        self.bead_offset_sb.setValue(50)
        self.bead_offset_sb.setSuffix('%')
        self.activate_path_pb = QtWidgets.QPushButton('Update')
        self.activate_path_pb.setToolTip('Segment into specified paths')
        
        #populate pathing layout
        path_gen_layout.addWidget(self.path_all_cb,0,0,1,1)
        path_gen_layout.addWidget(self.path_outline_cb,0,1,1,1)
        path_gen_layout.addWidget(self.rotate_z_path_sb,0,2,1,1)
        path_gen_layout.addWidget(self.by_width_sb,0,3,1,1)
        path_gen_layout.addWidget(self.bead_offset_sb,0,4,1,1)
        path_gen_layout.addWidget(self.num_paths_label,1,0,1,2)
        path_gen_layout.addWidget(self.activate_path_pb,1,4,1,1)
        
        self.path_box.setEnabled(False)
        
        #make outline modification box
        outline_mod_layout = QtWidgets.QGridLayout()
        outline_mod_box = collapsible_box('Outline modification')
        self.alpha_pb = QtWidgets.QPushButton("Apply")
        self.alpha_pb.setToolTip('Apply alpha shape smoothing to active profile.')
        self.alpha_undo_pb = QtWidgets.QPushButton("Revert")
        self.alpha_undo_pb.setToolTip('Revert to polygon winding')
        self.alpha_param_sb = QtWidgets.QDoubleSpinBox()
        self.alpha_param_sb.setPrefix('Alpha = ')
        self.alpha_param_sb.setMinimum(0.001)
        self.alpha_param_sb.setMaximum(1000)
        self.alpha_param_sb.setDecimals(3)
        self.alpha_param_sb.setValue(1)
        self.alpha_param_sb.setToolTip('Alpha shape semiperimeter threshold for concave outlines')
        
        
        #create figure canvas
        #initialize plot
        self.figure = plt.figure(figsize=(4,4))
        ax = plt.gca()
        ax.set_xlim([0, 1])
        ax.set_ylim([0, 1])
        img = plt.imread(importlib.resources.files('d3dslic3r') / 'meta/noslice.png')
        plt.text(0.5, 0.18, "No slice", ha='center', style='italic', fontweight = 'bold', color='darkgray', alpha=0.5, size= 18)
        plt.imshow(img, zorder=1, extent=[0.25, 0.75, 0.25, 0.75], alpha=0.5)
        plt.axis('off')
        #changes the background of the plot, otherwise white
        # self.figure.patch.set_facecolor((242/255,242/255,242/255))
        self.canvas = FigureCanvas(self.figure)
        toolbar = NavigationToolbar(self.canvas)
        self.canvas.setSizePolicy(sizePolicy)
        self.canvas.setMinimumSize(QtCore.QSize(500, 375)) #figsize(8,6)

        #add everything to the outline mod layout
        outline_mod_layout.addWidget(self.alpha_param_sb,0,0,1,1)
        outline_mod_layout.addWidget(self.alpha_undo_pb,0,1,1,1)
        outline_mod_layout.addWidget(self.alpha_pb,0,2,1,1)
        
        #add layouts to boxes & main widgets to main layouts
        self.slice_box.setLayout(slice_layout)
        self.path_box.setLayout(path_layout)
        outline_mod_box.set_content_layout(outline_mod_layout)
        path_layout.addLayout(path_gen_layout)
        path_layout.addWidget(self.canvas)
        path_layout.addWidget(toolbar)
        sep = QtWidgets.QFrame()
        sep.setFrameShape(QtWidgets.QFrame.HLine)
        sep.setFrameShadow(QtWidgets.QFrame.Sunken)
        sep.setSizePolicy(QtWidgets.QSizePolicy.Minimum,QtWidgets.QSizePolicy.Expanding)
        sep.setLineWidth(1)
        path_layout.addWidget(outline_mod_box)
        path_layout.addWidget(sep)
        
        
        lvlayout=QtWidgets.QVBoxLayout()
        lvlayout.addWidget(io_box)
        io_box.setLayout(io_layout)
        lvlayout.addWidget(self.geo_box)
        self.geo_box.set_content_layout(geo_button_layout)
        lvlayout.addWidget(self.slice_box)
        lvlayout.addWidget(self.path_box)

        lvlayout.addStretch(1)
        
        mainlayout.addWidget(self.vtkWidget)
        mainlayout.addStretch(1)
        mainlayout.addLayout(lvlayout)

        def initialize(self):
            self.vtkWidget.start()
            
class interactor(QtWidgets.QWidget):
    '''
    Inherits most properties from Qwidget, but primes the VTK window, and ties functions and methods to interactors defined in main_window
    '''
    def __init__(self,parent):
        super(interactor, self).__init__(parent)
        self.ui = main_window()
        self.ui.setup(self)
        self.ren = vtk.vtkRenderer()
        colors = vtk.vtkNamedColors()
        self.ren.SetBackground(colors.GetColor3d("white"))

        self.ui.vtkWidget.GetRenderWindow().AddRenderer(self.ren)
        self.iren = self.ui.vtkWidget.GetRenderWindow().GetInteractor()
        style=vtk.vtkInteractorStyleTrackballCamera()
        self.iren.SetInteractorStyle(style)
        self.iren.AddObserver("KeyPressEvent", self.keypress)
        # self.iren.AddObserver("MouseMoveEvent", self.on_mouse_move)
        self.ren.GetActiveCamera().ParallelProjectionOn()
        self.ui.vtkWidget.Initialize()
        
        make_logo(self.ren)
        
        self.picking = False
        self.slice_data = []
        
        self.ui.load_button.clicked.connect(self.load_stl)
        
        self.ui.op_slider.valueChanged[int].connect(self.change_opacity)
        self.ui.export_stl_button.clicked.connect(self.export_stl)
        self.ui.trans_widget.trans_origin_button.clicked.connect(self.apply_trans)
        self.ui.trans_reset_button.clicked.connect(self.reset_trans)
        self.ui.trans_widget.choose_vertex_button.clicked.connect(self.actuate_vertex_select)
        self.ui.trans_widget.first_centroid.clicked.connect(self.actuate_centroid_select)
        self.ui.rotation_widget.trans_origin_button.clicked.connect(self.apply_rotation)
        self.ui.scale_widget.trans_origin_button.clicked.connect(self.apply_scale)
        
        self.ui.update_slice_button.clicked.connect(self.do_slice)
        self.ui.slice_num_cb.currentIndexChanged.connect(self.draw_slice)
        self.ui.alpha_pb.clicked.connect(self.get_alpha_shape)
        self.ui.alpha_undo_pb.clicked.connect(self.undo_alpha_shape)
        self.ui.activate_path_pb.clicked.connect(self.make_paths)
        self.ui.export_slice.clicked.connect(self.export)
    
    def keypress(self, obj, event):
        '''
        VTK interactor-specific listener for keypresses
        '''
        key = obj.GetKeySym()
        
        if key =="1":
            xyview(self.ren)
        elif key =="2":
            yzview(self.ren)
        elif key =="3":
            xzview(self.ren)
        elif key == "Up":
            self.change_entry(1)
        elif key == "Down":
            self.change_entry(-1)
    
        self.ui.vtkWidget.update()
        
    def change_entry(self,val):
        """
        Handles logic for changing entries in the slice combobox based on keypress. Combobox changes call draw_slice
        """
        if self.ui.slice_num_cb.count() == 0:
            return
        
        if val < 0:
            if self.ui.slice_num_cb.currentIndex() == 0:
                self.ui.slice_num_cb.setCurrentIndex(self.ui.slice_num_cb.count()-1)
            else:
                self.ui.slice_num_cb.setCurrentIndex(self.ui.slice_num_cb.currentIndex()-1)
        else:
            if self.ui.slice_num_cb.currentIndex() == self.ui.slice_num_cb.count()-1:
                self.ui.slice_num_cb.setCurrentIndex(1)
            else:
                self.ui.slice_num_cb.setCurrentIndex(self.ui.slice_num_cb.currentIndex()+1)

        
    
    def load_stl(self):
        """
        Opens file dialog to get stl file, returns polydata and actor to self. Clears polydata and object_actor from attributes on successful load
        """
        
        
        filep = get_file('*.stl')
        
        if filep is None:
            return
        if not(os.path.isfile(filep)):
            print('Data file invalid.')
            return
        self.polydata = get_polydata_from_stl(filep)
        self.ui.load_label.setText(filep)
        self.trans = np.eye(4)
        
        self.ui.trans_widget.trans_origin_button.setEnabled(True)
        self.ui.rotation_widget.trans_origin_button.setEnabled(True)
        self.ui.scale_widget.trans_origin_button.setEnabled(True)
        self.ui.trans_reset_button.setEnabled(True)
        self.ui.trans_widget.choose_vertex_button.setEnabled(True)
        
        self.redraw_stl()


    def export_stl(self):
        '''
        Writes the current model geometry to a new STL file
        '''
        
        fileo = get_save_file('*.stl')
        if fileo is None:
            return
        writer = vtk.vtkSTLWriter()
        writer.SetFileName(fileo)
        writer.SetInputData(self.polydata)
        writer.Write()

    def redraw_stl(self):
        """
        Redraws the STL object: removes it if it exists then generates a new actor from the current polydata
        """
        if hasattr(self,'object_actor'):
            self.ren.RemoveActor(self.object_actor)
            self.ren.RemoveActor(self.origin_actor)
        if hasattr(self,'current_outline_actor'):
            self.ren.RemoveActor(self.current_outline_actor)
            self.ren.RemoveActor(self.outline_caption_actor)
            self.ui.figure.clear()
        if hasattr(self,'slice_actors'):
            self.ren.RemoveActor(self.slice_actors)
        else:
            self.ren.RemoveAllViewProps()
            self.ui.geo_box.setEnabled(True)
            self.ui.slice_box.setEnabled(True)
        
        self.object_actor = actor_from_polydata(self.polydata)
        self.ren.AddActor(self.object_actor)
        
        # make/update an origin actor
        self.origin_actor = vtk.vtkAxesActor()
        # change scale of origin on basis of size of stl_actor, only if this is the first entry
        origin_scale = np.max(self.object_actor.GetBounds())/16
        self.origin_actor.SetTotalLength(origin_scale,origin_scale,origin_scale)
        self.origin_actor.SetNormalizedShaftLength(1,1,1)
        self.origin_actor.SetNormalizedTipLength(0.2,0.2,0.2)
        
        self.origin_actor.GetXAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone()
        self.origin_actor.GetYAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone()
        self.origin_actor.GetZAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone()
        self.ren.AddActor(self.origin_actor)
        
        self.change_opacity(self.ui.op_slider.value())
        self.ren.ResetCamera()
        self.ui.vtkWidget.setFocus()
        self.ui.vtkWidget.update()
        

    def change_opacity(self,value):
        self.ui.op_slider_label.setText('Opacity: %d%%'%value)
        if hasattr(self,'object_actor'):
            self.object_actor.GetProperty().SetOpacity(value/100)
        self.ui.vtkWidget.update()


    def reset_trans(self):
        '''
        Applies the inverse of the current transformation matrix to revert all transformations, resets inputs for movement
        '''

        T = np.linalg.inv(self.trans)
        self.apply_transformation(T)

    def apply_trans(self):
        '''
        Applies the appropriate translation to the existing model object(s)
        '''
        self.ui.translate_drop_button.setChecked(False)
        
        T = np.eye(4)
        T[0,-1] = self.ui.trans_widget.translate_x.value()
        T[1,-1] = self.ui.trans_widget.translate_y.value()
        T[2,-1] = self.ui.trans_widget.translate_z.value()
        self.apply_transformation(T)
        if self.picking:
            self.actuate_vertex_select()

    def apply_rotation(self):
        '''
        Applies a rotation matrix to the current object
        '''
        
        T = get_trans_from_euler_angles( \
        self.ui.rotation_widget.rotate_x.value(), \
        self.ui.rotation_widget.rotate_y.value(), \
        self.ui.rotation_widget.rotate_z.value())
        self.apply_transformation(T)

    def apply_scale(self):
        
        T = np.eye(4)
        
        if self.ui.scale_widget.scale_uniform_cb.isChecked():
            #Get maximum of entries
            sfactor = np.max([
            self.ui.scale_widget.scale_x.value(),
            self.ui.scale_widget.scale_y.value(),
            self.ui.scale_widget.scale_z.value()
            ])
            self.ui.scale_widget.scale_x.setValue(sfactor)
            self.ui.scale_widget.scale_y.setValue(sfactor)
            self.ui.scale_widget.scale_z.setValue(sfactor)
        
        T[0,0] = self.ui.scale_widget.scale_x.value()
        T[1,1] = self.ui.scale_widget.scale_y.value()
        T[2,2] = self.ui.scale_widget.scale_z.value()
        self.apply_transformation(T)


    def apply_transformation(self,T):
        '''
        Applies transformation matrix T to current STL entry
        '''
        
        #modify relevant aspects of the instance according to the transformation matrix
        np_pts = do_transform(v2n(self.polydata.GetPoints().GetData()),T)
        self.trans = T @ self.trans
        c_points = self.polydata.GetPoints()
        for i in range(len(np_pts)):
            c_points.SetPoint(i, np_pts[i,:])
        self.polydata.Modified()
        

        self.redraw_stl()
        self.ren.ResetCamera()
        self.ui.vtkWidget.update()
        # self.ui.vtkWidget.setFocus()
    
    def actuate_centroid_select(self):
        '''
        Populates translate widget with centroid coordinates of the first slice
        '''
        try:
            cent = - np.mean(self.slice_data[0].outline, axis=0)
            self.ui.trans_widget.translate_x.setValue(cent[0])
            self.ui.trans_widget.translate_y.setValue(cent[1])
            self.ui.trans_widget.translate_z.setValue(cent[2])
        except:
            print('Finding the centroid of the first slice did not happen.')
            return
        
        
    
    def actuate_vertex_select(self):
        '''
        Starts picking and handles ui button display
        '''
        
        #selected actor is the vertex highlight
        if hasattr(self,'selected_actor'):
            self.ren.RemoveActor(self.selected_actor)
        
        if self.picking:
            #Remove picking observer and re-initialise
            self.iren.RemoveObservers('LeftButtonPressEvent')
            self.iren.AddObserver('LeftButtonPressEvent',self.default_left_button)
            QtWidgets.QApplication.processEvents()
            self.picking = False
            self.ui.translate_drop_button.setChecked(False)
            self.ui.trans_widget.choose_vertex_button.setChecked(False)

        else:
            self.iren.AddObserver('LeftButtonPressEvent', self.picker_callback)
            self.picking = True
            #meant to keep dropdown engaged through the picking process, but ineffective. Stopping picking suspends, as does 'updating'.
            self.ui.trans_widget.choose_vertex_button.setChecked(True)
            self.ui.translate_drop_button.setChecked(True)

    def default_left_button(self, obj, event):
        #forward standard events according to the default style`
        self.iren.GetInteractorStyle().OnLeftButtonDown()

    def picker_callback(self, obj, event):
        """
        Actuates a pick of a node on current component
        """
        

        colors = vtk.vtkNamedColors()
        
        picker = vtk.vtkPointPicker()
        picker.SetTolerance(1)
        
        pos = self.iren.GetEventPosition()
        
        picker.Pick(pos[0], pos[1], 0, self.ren)

        if picker.GetPointId() != -1:
            
            ids = vtk.vtkIdTypeArray()
            ids.SetNumberOfComponents(1)
            ids.InsertNextValue(picker.GetPointId())

            if hasattr(self,'selected_actor'):
                self.ren.RemoveActor(self.selected_actor)
            centre = self.polydata.GetPoint(picker.GetPointId())
            self.selected_actor = generate_sphere(centre,1,colors.GetColor3d("orchid"))
            
            self.ui.trans_widget.translate_x.setValue(-centre[0])
            self.ui.trans_widget.translate_y.setValue(-centre[1])
            self.ui.trans_widget.translate_z.setValue(-centre[2])
            
            self.ren.AddActor(self.selected_actor)

    def do_slice(self):
        """
        Performs slicing operation based on ui values
        """
        
        if hasattr(self,'slice_actors'):
            self.ren.RemoveActor(self.slice_actors)
            self.ui.slice_num_cb.clear()
            del self.slice_data
        
        
        if self.ui.quantity_rb.isChecked():
            self.outlines, self.slice_actors = get_slice_data(self.polydata,self.ui.by_num_sb.value())
        else:
            self.outlines, self.slice_actors = get_slice_data(self.polydata,self.ui.by_height_sb.value(), False)
        
        self.outlines = get_sub_slice_data(self.outlines,self.ui.agglom_param_sb.value())
        
        #order points
        for i in range(len(self.outlines)):
            self.outlines[i] = order_points_in_loop(self.outlines[i])
            
        self.slice_data = [None] * len(self.outlines)
        
        for i in range(len(self.outlines)):
            self.ui.slice_num_cb.insertItem(i,'Slice %d'%i)
        self.ui.slice_num_cb.setEnabled(True)
        
        self.ui.path_box.setEnabled(True)
        self.ui.trans_widget.first_centroid.setEnabled(True)
        
        self.ren.AddActor(self.slice_actors)
        self.ui.vtkWidget.update()

    def draw_slice(self):
        
        #get active outline
        entry = self.ui.slice_num_cb.currentIndex()
        
        #make sure inadvertent changes to combobox don't operate
        if not self.slice_data:
            return
        
        if hasattr(self,'current_outline_actor'):
            self.ren.RemoveActor(self.current_outline_actor)
            self.ren.RemoveActor(self.outline_caption_actor)
            self.ui.figure.clear()
        
        #get active outline
        entry = self.ui.slice_num_cb.currentIndex()
        
        #check if there is itemData for the current entry, initialize a slice_obj otherwise
        if self.slice_data[entry] is None:
            self.slice_data[entry] = slice_obj(self.outlines[entry])
        
        outline = self.slice_data[entry].outline
        
        self.current_outline_actor = gen_outline_actor(outline, (1,0,0), 4)
        self.outline_caption_actor = gen_caption_actor('%s'%entry, self.current_outline_actor, (1,0,0))
        
        self.ren.AddActor(self.current_outline_actor)
        self.ren.AddActor(self.outline_caption_actor)
        
        #add to combobox item data
        
        self.ui.vtkWidget.update()
        
        ax = self.ui.figure.add_subplot(111)
        ax.plot(outline[:,0], outline[:,1], 'k-')
        ax.set_ylabel("y (mm)")
        ax.set_xlabel("x (mm)")
        ax.grid(visible=True, which='major', color='#666666', linestyle='-', alpha=0.1)
        ax.minorticks_on()
        ax.grid(visible=True, which='minor', color='#666666', linestyle='-', alpha=0.2)
        ax.set_aspect('equal', adjustable='datalim')
        
        ax.text(0.95, 0.01, 'z = %0.3f'%np.mean(outline[:,2]),
        verticalalignment='bottom', horizontalalignment='right',
        transform=ax.transAxes,
        color='green', fontsize=12)
        
        self.ui.canvas.draw()
        
        #call draw_paths in case there are paths available
        self.draw_paths()
        
    def get_alpha_shape(self):
        """
        Gets parameters from GUI, runs sort_alpha and returns the result.
        """
        
        entry = self.ui.slice_num_cb.currentIndex()
        
        #check if alpha shape processing is on, if it is then call get_alpha_shape on this outline
        alpha_outline = sort_alpha(self.slice_data[entry].outline, self.ui.alpha_param_sb.value())
        if alpha_outline is None:
            print('No alpha')
            pass
        else:
            self.slice_data[entry] = slice_obj(alpha_outline)
        
        self.draw_slice()
    
    def undo_alpha_shape(self):
        """
        Returns profile to raw, sort_ccw form
        """
        
        entry = self.ui.slice_num_cb.currentIndex()
        self.slice_data[entry] = slice_obj(self.outlines[entry])
        
        self.draw_slice()
        
    def make_paths(self):
        """
        Updates the slice_obj for each slice to include a 'paths' entry. Calls draw_paths to draw them.
        """
        
        local_path_collection = []
        
        if not self.ui.path_all_cb.isChecked():
        
            entry = self.ui.slice_num_cb.currentIndex()
            outline = self.slice_data[entry].outline
            theta = self.ui.rotate_z_path_sb.value()
            offset = self.ui.bead_offset_sb.value()/100
            if self.ui.path_outline_cb.isChecked():
                interior = offset_poly(outline,-self.ui.by_width_sb.value())
                midline = offset_poly(outline,-self.ui.by_width_sb.value()*0.5)
                intersections, param = get_intersections(midline,theta,self.ui.by_width_sb.value(),offset)
                local_path_collection.append(midline)
            else:
                intersections, param = get_intersections(outline,theta,self.ui.by_width_sb.value(),offset)

            self.ui.bead_offset_sb.setValue(param*100)
            self.ui.num_paths_label.setText('N = %i'%(len(intersections)-1))
            
        #pack up/pair intersections for line paths
        for i in np.arange(len(intersections)-1)[::2]:
            local_path_collection.append(np.array([intersections[i,:], intersections[i+1,:]]))
            
        
        self.slice_data[entry].paths = local_path_collection #list of paths
        self.ui.export_slice.setEnabled(True)
        self.draw_slice() #to clear any existing intersections

    def draw_paths(self):

        #get active outline
        entry = self.ui.slice_num_cb.currentIndex()
        
        #make sure there's something to plot
        if not self.slice_data[entry].paths:
            return
        
        ax = self.ui.figure.gca()
        for path in self.slice_data[entry].paths:
            ax.plot(path[:,0], path[:,1], '--', color = 'k')
    
        self.ui.canvas.draw()

    def export(self):
        '''
        Create pop-up which shows the user what's going to happen
        '''
        ew = export_widget(self, self.slice_data)
        ew.exec_()

        

class slice_obj:
    def __init__(self, outline):
        self.outline = outline
        self.alpha = None
        self.paths = None

if __name__ == "__main__":
    launch()


#!/usr/bin/env python
'''
d3dslic3r export_widget - popup to allow for directing export formats
'''

__author__ = "M.J. Roy"
__version__ = "0.1"
__email__ = "matthew.roy@manchester.ac.uk"
__status__ = "Experimental"
__copyright__ = "(c) M. J. Roy, 2014--"

import os, io
import subprocess as sp
import numpy as np

from PyQt5 import QtGui, QtWidgets, QtCore, QtSvg
from PyQt5.QtCore import Qt
from pkg_resources import Requirement, resource_filename

class export_widget(QtWidgets.QDialog):

    def __init__(self, parent, slice_data):
        super(export_widget, self).__init__(parent)
        self.slice_data = slice_data

        self.setWindowTitle("d3dslic3r - export")
        self.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
        self.setMinimumSize(QtCore.QSize(450, 200))
        
        self.clip_label = QtWidgets.QLabel()
        self.export_status_label = QtWidgets.QLabel()
        pix_map1 = QtGui.QPixmap(resource_filename("d3dslic3r","meta/clippy.png"))
        self.clip_label.setPixmap(pix_map1)

        param_layout = QtWidgets.QGridLayout()
        
        desc = QtWidgets.QLabel("Hmm. It looks like you're exporting some slices and paths. The format will be whatever is set as the prefix, with an integer for the slice number, followed by the path number. For example, if the prefix is 'Slice', then the 3rd path of the 4th slice would be 'Slice_3_2.txt'.")
        desc.setWordWrap(True) 
        
        prefix_label = QtWidgets.QLabel("File prefix:")
        self.prefix = QtWidgets.QLineEdit("Slice")
        
        work_dir_path_label = QtWidgets.QLabel('Working directory:')
        self.work_dir_path = QtWidgets.QLineEdit()
        wd_choose_path = QtWidgets.QPushButton('...')
        wd_choose_path.setMaximumWidth(60)
        wd_choose_path.setAutoDefault(False)
        
        export = QtWidgets.QPushButton('Execute')
        
        param_layout.addWidget(prefix_label,0,0,1,1)
        param_layout.addWidget(self.prefix,0,1,1,2)
        param_layout.addWidget(export,0,3,1,1)
        param_layout.addWidget(work_dir_path_label,1,0,1,1)
        param_layout.addWidget(self.work_dir_path,1,1,1,2)
        param_layout.addWidget(wd_choose_path,1,3,1,1)
        
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addWidget(desc)
        main_layout.addLayout(param_layout)
        

        clips_layout = QtWidgets.QHBoxLayout()
        clips_layout.addWidget(self.export_status_label)
        clips_layout.addStretch(1)
        clips_layout.addWidget(self.clip_label)
        
        # clips_layout.addWidget(svg2)
        main_layout.addLayout(clips_layout)
        
        wd_choose_path.clicked.connect(self.set_wd)
        export.clicked.connect(self.export)
        
        self.setLayout(main_layout)
        self.show()

    def set_wd(self):
        dir = str(QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory"))
        if dir != '':
            self.work_dir_path.setText(dir)
        else:
            return
    
    def export(self):
        """
        If the current working directory is valid, write out all of the entries of each slice_obj in slice_data to txt files with np.genfromtxt
        """
        
        if not os.path.isdir(self.work_dir_path.text()):
            return
        
        i = 0
        for slice_obj in self.slice_data:
            
            if slice_obj is not None:
                j = 0
                if slice_obj.paths is not None:
                    for path in slice_obj.paths:
                        np.savetxt(os.path.join(self.work_dir_path.text(),"%s_%i_%i.txt"%(self.prefix.text(),i,j)),path)
                        j+=1
            i+=1
        
        pix_map2 = QtGui.QPixmap(resource_filename("d3dslic3r","meta/dippy.png"))
        self.clip_label.setPixmap(pix_map2)
        self.export_status_label.setText("Complete.")
        self.clip_label.update()
        
if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    w = export_widget(None, None)
    sys.exit(app.exec_())


import os
import vtk

from PyQt5 import QtCore, QtGui, QtWidgets
from vtk.util.numpy_support import numpy_to_vtk as n2v
import importlib.resources

class collapsible_box(QtWidgets.QWidget):
    def __init__(self, title="", parent=None):
        super(collapsible_box, self).__init__(parent)

        self.toggle_button = QtWidgets.QToolButton(
            text=title, checkable=True, checked=False
        )
        self.toggle_button.setStyleSheet("QToolButton { border: none; }")
        self.toggle_button.setToolButtonStyle(
            QtCore.Qt.ToolButtonTextBesideIcon
        )
        self.toggle_button.setArrowType(QtCore.Qt.RightArrow)
        self.toggle_button.pressed.connect(self.on_pressed)

        self.toggle_animation = QtCore.QParallelAnimationGroup(self)

        self.content_area = QtWidgets.QScrollArea(
            maximumHeight=0, minimumHeight=0
        )
        self.content_area.setSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed
        )
        self.content_area.setFrameShape(QtWidgets.QFrame.NoFrame)

        lay = QtWidgets.QVBoxLayout(self)
        lay.setSpacing(0)
        lay.setContentsMargins(0, 0, 0, 0)
        lay.addWidget(self.toggle_button)
        lay.addWidget(self.content_area)

        self.toggle_animation.addAnimation(
            QtCore.QPropertyAnimation(self, b"minimumHeight")
        )
        self.toggle_animation.addAnimation(
            QtCore.QPropertyAnimation(self, b"maximumHeight")
        )
        self.toggle_animation.addAnimation(
            QtCore.QPropertyAnimation(self.content_area, b"maximumHeight")
        )

    @QtCore.pyqtSlot()
    def on_pressed(self):
        checked = self.toggle_button.isChecked()
        self.toggle_button.setArrowType(
            QtCore.Qt.DownArrow if not checked else QtCore.Qt.RightArrow
        )
        self.toggle_animation.setDirection(
            QtCore.QAbstractAnimation.Forward
            if not checked
            else QtCore.QAbstractAnimation.Backward
        )
        self.toggle_animation.start()

    def set_content_layout(self, layout):
        lay = self.content_area.layout()
        del lay
        self.content_area.setLayout(layout)
        collapsed_height = (
            self.sizeHint().height() - self.content_area.maximumHeight()
        )
        content_height = layout.sizeHint().height()
        for i in range(self.toggle_animation.animationCount()):
            animation = self.toggle_animation.animationAt(i)
            animation.setDuration(500)
            animation.setStartValue(collapsed_height)
            animation.setEndValue(collapsed_height + content_height)

        content_animation = self.toggle_animation.animationAt(
            self.toggle_animation.animationCount() - 1
        )
        content_animation.setDuration(500)
        content_animation.setStartValue(0)
        content_animation.setEndValue(content_height)

def make_splash():
    '''
    Makes and returns a Qt splash window object
    '''
    spl_fname = importlib.resources.files('d3dslic3r') / 'meta/Logo.png'
    splash_pix = QtGui.QPixmap(spl_fname.__str__(),'PNG')
    splash = QtWidgets.QSplashScreen(splash_pix, QtCore.Qt.SplashScreen)
    splash.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint | QtCore.Qt.FramelessWindowHint)
    
    font = splash.font()
    font.setPixelSize(20)
    font.setWeight(QtGui.QFont.Bold)
    splash.setFont(font)
    
    # splash.showMessage('v%s'%(version('d3dslic3r')),QtCore.Qt.AlignRight | QtCore.Qt.AlignBottom, QtCore.Qt.lightGray)
    splash.showMessage('v%s'%("0.0.1"),QtCore.Qt.AlignRight | QtCore.Qt.AlignTop, QtCore.Qt.darkGray)
    return splash

def make_logo(ren):
    spl_fname = importlib.resources.files('d3dslic3r') / 'meta/background.png'
    img_reader = vtk.vtkPNGReader()
    img_reader.SetFileName(spl_fname)
    img_reader.Update()
    logo = vtk.vtkLogoRepresentation()
    logo.SetImage(img_reader.GetOutput())
    logo.ProportionalResizeOn()
    logo.SetPosition( 0.1, 0.1 ) #lower left
    logo.SetPosition2( 0.8, 0.8 ) #upper right
    logo.GetImageProperty().SetDisplayLocationToBackground()
    ren.AddViewProp(logo)
    logo.SetRenderer(ren)
    return logo

def get_file(*args):
    '''
    Returns absolute path to filename and the directory it is located in from a PyQt5 filedialog. First value is file extension, second is a string which overwrites the window message.
    '''
    ext = args[0]
    if len(args)>1:
        id = args[1]
    else: id = os.getcwd()
    ftypeName={}
    ftypeName['*.stl']=["STereoLithography file", "*.stl","STL file"]
    ftypeName['*.*'] = ["d3dslic3r external executable", "*.*", "..."]
    
    
    filer = QtWidgets.QFileDialog.getOpenFileName(None, str(ftypeName[ext][0]), 
         id,(ext))

    if filer[0] == '':
        return None
        
    else: #return the filename/path
        return filer[0]

def get_save_file(*args):
    '''
    Gets a file path for saving
    '''

    ftypeName={}
    ftypeName['*.stl']='STereoLithography file'
    
    ext = args[0]
    if len(args)>1:
        id = args[1]
    else: id = os.getcwd()
    
    filer, _ = QtWidgets.QFileDialog.getSaveFileName(None, "Save as:", id,str(ftypeName[ext]+' ('+ext+')'))
    
    if filer == '':
        return None
    else:
        return filer

def gen_outline_actor(pts, color = (1,1,1), size = 2):
    '''
    Returns an outline actor with specified numpy array of points, color and size. pts should be ordered
    '''
    if color[0]<=1 and color != None:
        color=(int(color[0]*255),int(color[1]*255),int(color[2]*255))
    if color[0]>=1 and color != None:
        color=(color[0]/float(255),color[1]/float(255),color[2]/float(255))
    points=vtk.vtkPoints()

    points.SetData(n2v(pts))

    lineseg=vtk.vtkPolygon()
    lineseg.GetPointIds().SetNumberOfIds(len(pts))
    for i in range(len(pts)):
        lineseg.GetPointIds().SetId(i,i)
    linesegcells=vtk.vtkCellArray()
    linesegcells.InsertNextCell(lineseg)
    outline=vtk.vtkPolyData()
    outline.SetPoints(points)
    outline.SetVerts(linesegcells)
    outline.SetLines(linesegcells)
    mapper=vtk.vtkPolyDataMapper()
    mapper.SetInputData(outline)
    outline_actor=vtk.vtkActor()
    outline_actor.SetMapper(mapper)
    outline_actor.GetProperty().SetColor(color)
    outline_actor.GetProperty().SetPointSize(size)
    return outline_actor

def gen_caption_actor(message, actor = None, color = (0,0,0)):
    '''
    Captions an actor
    '''
    caption_actor = vtk.vtkCaptionActor2D()
    b = actor.GetBounds()
    caption_actor.SetAttachmentPoint((b[0],b[2],b[4]))
    caption_actor.SetCaption(message)
    caption_actor.SetThreeDimensionalLeader(False)
    caption_actor.BorderOff()
    caption_actor.LeaderOff()
    caption_actor.SetWidth(0.25 / 3.0)
    caption_actor.SetHeight(0.10 / 3.0)
    
    p = caption_actor.GetCaptionTextProperty()
    p.SetColor(color)
    p.BoldOn()
    p.ItalicOff()
    p.SetFontSize(36)
    p.ShadowOn()
    return caption_actor

def generate_sphere(center, radius, color):
    source = vtk.vtkSphereSource()
    source.SetCenter(*center)
    source.SetRadius(radius)
    source.SetThetaResolution(20)
    source.SetPhiResolution(20)
    source.Update()
    
    mapper = vtk.vtkPolyDataMapper()
    mapper.SetInputData(source.GetOutput())
    
    actor = vtk.vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetColor(*color)
    
    return actor

def xyview(ren):
    camera = ren.GetActiveCamera()
    camera.SetPosition(0,0,1)
    camera.SetFocalPoint(0,0,0)
    camera.SetViewUp(0,1,0)
    ren.ResetCamera()

def yzview(ren):
    camera = ren.GetActiveCamera()
    camera.SetPosition(1,0,0)
    camera.SetFocalPoint(0,0,0)
    camera.SetViewUp(0,0,1)
    ren.ResetCamera()

def xzview(ren):
    vtk.vtkObject.GlobalWarningDisplayOff() #mapping from '3' triggers an underlying stereoview that most displays do not support for trackball interactors
    camera = ren.GetActiveCamera()
    camera.SetPosition(0,1,0)
    camera.SetFocalPoint(0,0,0)
    camera.SetViewUp(0,0,1)
    ren.ResetCamera()


#!/usr/bin/env python
'''
pyCM fea_widget - runs external thread to run an FEA from a pyCM output file.
'''

__author__ = "M.J. Roy"
__version__ = "0.1"
__email__ = "matthew.roy@manchester.ac.uk"
__status__ = "Experimental"
__copyright__ = "(c) M. J. Roy, 2014--"

import os, io
import subprocess as sp
import numpy as np
import vtk
import vtk.util.numpy_support as v2n
from PyQt5 import QtGui, QtWidgets, QtCore
from PyQt5.QtCore import Qt, QThread, pyqtSignal
import yaml
from d3dslic3r.d3dslic3r_gui_common import get_file, get_save_file
import importlib.resources

class execute(QThread):
    '''
    Sets up and runs external thread, emits 100 when done.
    '''
    _signal = pyqtSignal(int)
    def __init__(self,input_file,exe,val,direction):
        super(execute, self).__init__()
        #variables passed here
        self.input_file = input_file
        self.exe = exe #executable path
        self.which_dir = direction
        self.cut_val = val

    def run(self):
        current_dir = os.getcwd()
        output_dir = os.path.dirname(self.input_file)
        base = os.path.basename(self.input_file)
        os.chdir(output_dir)
        
        try:
            print('Slic3r console exec: %s -i %s'%(self.exe,base))
        
            out=sp.check_output([self.exe,"--cut",str(self.cut_val),base], shell=True)
            print("Slic3r output log:")
            print("----------------")
            print(out.decode("utf-8"))
            print("----------------")
            print("d3dslic3r: Slic3r run completed . . . Idle")
        except sp.CalledProcessError as e:
            print("Slic3r command failed for some reason.")
            print(e)
        
        self._signal.emit(100)
        os.chdir(current_dir)
        
    
class slic3r_widget(QtWidgets.QDialog):

    def __init__(self, parent, file):
        super(slic3r_widget, self).__init__(parent)
        self.file = file

        self.setWindowTitle("d3dslic3r using Slic3r: %s"%os.path.basename(self.file))
        self.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
        self.setMinimumSize(QtCore.QSize(450, 200))

        param_layout = QtWidgets.QGridLayout()
        
        self.cut_x_rb=QtWidgets.QRadioButton("Cut X")
        self.cut_x_rb.setEnabled(False)
        self.cut_y_rb=QtWidgets.QRadioButton("Cut Y")
        self.cut_y_rb.setEnabled(False)
        self.cut_z_rb=QtWidgets.QRadioButton("Cut Z")
        self.cut_val=QtWidgets.QDoubleSpinBox()
        self.cut_z_rb.setChecked(True)

        mtype_button_group = QtWidgets.QButtonGroup()
        mtype_button_group.addButton(self.cut_x_rb)
        mtype_button_group.addButton(self.cut_y_rb)
        mtype_button_group.addButton(self.cut_z_rb)
        mtype_button_group.setExclusive(True)
        
        self.clean_up_cb = QtWidgets.QCheckBox('Remove intermediate files')
        self.clean_up_cb.setToolTip('Remove intermediate files after running command')
        self.clean_up_cb.setChecked(False)
        self.clean_up_cb.setEnabled(False)
        
        param_layout.addWidget(self.cut_x_rb,0,0,1,1)
        param_layout.addWidget(self.cut_y_rb,0,1,1,1)
        param_layout.addWidget(self.cut_z_rb,0,2,1,1)
        param_layout.addWidget(self.cut_val,0,3,1,1)
        param_layout.addWidget(self.clean_up_cb,0,4,1,1)
        
        self.pbar = QtWidgets.QProgressBar(self, textVisible=True)
        self.pbar.setAlignment(Qt.AlignCenter)
        self.pbar.setFormat("Idle")
        self.pbar.setFont(QtGui.QFont("Helvetica",italic=True))
        self.pbar.setValue(0)

        self.run_button = QtWidgets.QPushButton('Run')
        slic3r_exec_path_label = QtWidgets.QLabel('Slic3r console executable:')
        self.slic3r_exec_path = QtWidgets.QLineEdit()
        slic3r_choose_path = QtWidgets.QPushButton('...')
        slic3r_choose_path.setMaximumWidth(30)
        slic3r_choose_path.setAutoDefault(False)
        work_dir_path_label = QtWidgets.QLabel('Working directory:')
        self.work_dir_path = QtWidgets.QLineEdit()
        wd_choose_path = QtWidgets.QPushButton('...')
        wd_choose_path.setMaximumWidth(30)
        wd_choose_path.setAutoDefault(False)

        run_layout = QtWidgets.QGridLayout()
        
        run_layout.addWidget(slic3r_exec_path_label,1,0,1,1)
        run_layout.addWidget(self.slic3r_exec_path,1,1,1,2)
        run_layout.addWidget(slic3r_choose_path,1,3,1,1)
        
        run_layout.addWidget(work_dir_path_label,2,0,1,1)
        run_layout.addWidget(self.work_dir_path,2,1,1,2)
        run_layout.addWidget(wd_choose_path,2,3,1,1)
        run_layout.addWidget(self.run_button,3,0,1,1)
        run_layout.addWidget(self.pbar,3,1,1,3)

        self.run_button.clicked.connect(self.run_slic3r)
        slic3r_choose_path.clicked.connect(self.set_slic3r)
        wd_choose_path.clicked.connect(self.set_wd)
        
        self.layout = QtWidgets.QVBoxLayout()
        self.layout.addLayout(param_layout)
        self.layout.addLayout(run_layout)

        self.setLayout(self.layout)
        self.read_config()
        self.show()

    
    
    def run_slic3r(self):
        self.make_config_change() #save anything that the user might have put into config boxes
        
        self.input_file = get_file("*.stl", self.work_dir_path.text())
        if self.input_file is None:
            return
        
        

        self.thread = execute(self.input_file,self.slic3r_exec_path.text(),        self.cut_val.value(), None) #None should be direction

        self.thread._signal.connect(self.signal_accept)
        self.thread.start()
        self.pbar.setTextVisible(True)
        self.pbar.setStyleSheet("")
        self.pbar.setRange(0,0)
        
    def signal_accept(self, msg):
        if int(msg) == 100:
            self.pbar.setRange(0,100)
            self.pbar.setValue(0)
            self.pbar.setFormat("Complete")
            self.pbar.setStyleSheet("QProgressBar"
              "{"
              "background-color: lightgreen;"
              "border : 1px"
              "}")
    
    def set_slic3r(self):
        f = get_file("*.*")
        if f is None or not(os.path.isfile(f)):
            return
        self.slic3r_exec_path.setText(f)
        self.make_config_change()

    def set_wd(self):
        dir = str(QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory"))
        if dir != '':
            self.work_dir_path.setText(dir)
            self.make_config_change()
        else:
            return

    def read_config(self):
        fname=importlib.resources.files('d3dslic3r') / 'meta/d3dslic3r_config.yml'
        with open(fname, 'r') as f:
            read = yaml.safe_load(f)
        
        self.slic3r_exec_path.setText(read['slic3r']['exec'])
        self.work_dir_path.setText(read['slic3r']['work_dir'])


    def make_config_change(self):
        new_entries = dict(
        slic3r_exec = str(self.slic3r_exec_path.text()),
        work_dir = str(self.work_dir_path.text())
        )
        
        fname=importlib.resources.files('d3dslic3r') / 'meta/d3dslic3r_config.yml'
        with open(fname, 'r') as yamlfile:
            cur_yaml = yaml.safe_load(yamlfile)
            cur_yaml['slic3r'].update(new_entries)
        if cur_yaml:
            with open(fname, 'w') as yamlfile:
                yaml.safe_dump(cur_yaml, yamlfile)

    def closeEvent(self, event):
        '''
        Not implemented
        '''
        pass


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    if len(sys.argv)>1:
        fw = slic3r_widget(None, sys.argv[1])
    else:
        fw = slic3r_widget(None, 'No file specified')
    sys.exit(app.exec_())


#!/usr/bin/env python
'''
Constructor functions/methods for transformation widgets. make_x_button could be abstracted.
'''

import sys
import numpy as np
from PyQt5 import QtCore, QtGui, QtWidgets
import importlib.resources


def make_transformation_button_layout(parent_window):
    '''
    Makes a button group that contains a translation and reset button
    '''
    button_layout = QtWidgets.QHBoxLayout()
    parent_window.trans_reset_button = QtWidgets.QPushButton('Reset')
    parent_window.trans_reset_button.setToolTip('Reset all current transformations')
    parent_window.trans_reset_button.setEnabled(False)
    parent_window.translate_drop_button = make_translate_button(parent_window)
    button_layout.addWidget(parent_window.translate_drop_button)
    parent_window.rotate_drop_button = make_rotate_button(parent_window)
    button_layout.addWidget(parent_window.rotate_drop_button)
    parent_window.scale_drop_button = make_scale_button(parent_window)
    button_layout.addWidget(parent_window.scale_drop_button)
    button_layout.addWidget(parent_window.trans_reset_button)
    button_layout.addStretch(1)
    return button_layout


def make_translate_button(parent_window):
    ico = importlib.resources.files('d3dslic3r') / 'meta/translate_icon.png'
    with importlib.resources.as_file(ico) as path:
        translate_icon = QtGui.QIcon(QtGui.QIcon(path.__str__()))
    translate_drop_button = QtWidgets.QToolButton()
    translate_drop_button.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
    translate_drop_button.setCheckable(True)
    translate_drop_button.setMenu(QtWidgets.QMenu(translate_drop_button))
    trans_action = QtWidgets.QWidgetAction(translate_drop_button)
    parent_window.trans_widget = transform_box(parent_window,'translate')
    trans_action.setDefaultWidget(parent_window.trans_widget)
    translate_drop_button.menu().addAction(trans_action)
    translate_drop_button.setIcon(translate_icon)
    translate_drop_button.setToolTip('Translate to new origin')
    return translate_drop_button

def make_rotate_button(parent_window):
    ico = importlib.resources.files('d3dslic3r') / 'meta/rotate_icon.png'
    with importlib.resources.as_file(ico) as path:
        rotate_icon = QtGui.QIcon(QtGui.QIcon(path.__str__()))
    rotate_drop_button = QtWidgets.QToolButton()
    rotate_drop_button.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
    rotate_drop_button.setCheckable(True)
    rotate_drop_button.setMenu(QtWidgets.QMenu(rotate_drop_button))
    trans_action = QtWidgets.QWidgetAction(rotate_drop_button)
    parent_window.rotation_widget = transform_box(parent_window, 'rotate')
    trans_action.setDefaultWidget(parent_window.rotation_widget)
    rotate_drop_button.menu().addAction(trans_action)
    rotate_drop_button.setIcon(rotate_icon)
    rotate_drop_button.setToolTip('Rotate about origin')
    return rotate_drop_button
    
def make_scale_button(parent_window):
    ico = importlib.resources.files('d3dslic3r') / 'meta/scale_icon.png'
    with importlib.resources.as_file(ico) as path:
        scale_icon = QtGui.QIcon(QtGui.QIcon(path.__str__()))
    scale_drop_button = QtWidgets.QToolButton()
    scale_drop_button.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
    scale_drop_button.setCheckable(True)
    scale_drop_button.setMenu(QtWidgets.QMenu(scale_drop_button))
    trans_action = QtWidgets.QWidgetAction(scale_drop_button)
    parent_window.scale_widget = transform_box(parent_window, 'scale')
    trans_action.setDefaultWidget(parent_window.scale_widget)
    scale_drop_button.menu().addAction(trans_action)
    scale_drop_button.setIcon(scale_icon)
    scale_drop_button.setToolTip('Scale over main axes')
    return scale_drop_button

def get_trans_from_euler_angles(ax,ay,az):
    '''
    Based on incoming arguments in *degrees*, return a 4x4 transformation matrix
    '''
    ax = np.deg2rad(ax)
    Rx = np.array([[1,0,0],[0, np.cos(ax), -np.sin(ax)],[0, np.sin(ax), np.cos(ax)]])
    ay = np.deg2rad(ay)
    Ry = np.array([[np.cos(ay), 0, np.sin(ay)],[0,1,0],[-np.sin(ay), 0, np.cos(ay)]])
    az = np.deg2rad(az)
    Rz = np.array([[np.cos(az), -np.sin(az), 0],[np.sin(az), np.cos(az), 0],[0,0,1]])
    R = Rx @ Ry @ Rz
    
    trans = np.identity(4)
    trans[0:3,0:3] = R
    return trans

class transform_box(QtWidgets.QWidget):
    def __init__(self, parent_window, cond, *args, **kwargs):
        '''
        Makes a complete reorientation widget
        '''
        
        super().__init__(*args, **kwargs)
        
        self.setSizePolicy(
            QtWidgets.QSizePolicy.MinimumExpanding,
            QtWidgets.QSizePolicy.MinimumExpanding
        )
        
        vl = QtWidgets.QVBoxLayout()
        hl = QtWidgets.QHBoxLayout()
        button_layout = QtWidgets.QHBoxLayout()
        translate_box = QtWidgets.QGroupBox('Translate:')
        translate_layout = QtWidgets.QVBoxLayout()
        rotate_box = QtWidgets.QGroupBox('Rotate about:')
        rotate_layout = QtWidgets.QVBoxLayout()
        scale_box = QtWidgets.QGroupBox('Scale over:')
        scale_layout = QtWidgets.QVBoxLayout()
        
        
        self.setLayout(button_layout)
        
        #right hand button group
        self.choose_vertex_button = QtWidgets.QPushButton('Vertex')
        self.choose_vertex_button.setEnabled(False)
        self.choose_vertex_button.setCheckable(True)
        self.choose_vertex_button.setToolTip("Select vertex from viewport")
        self.first_centroid = QtWidgets.QPushButton('Centroid')
        self.first_centroid.setToolTip("Select centroid of first slice")
        self.first_centroid.setEnabled(False)
        
        self.trans_origin_button = QtWidgets.QPushButton('Update')
        self.trans_origin_button.setToolTip('Apply transformation')
        self.trans_origin_button.setEnabled(False)
        
        self.translate_x = QtWidgets.QDoubleSpinBox()
        self.translate_x.setMinimum(-1000)
        self.translate_x.setValue(0)
        self.translate_x.setMaximum(1000)
        self.translate_x.setPrefix('X ')
        self.translate_x.setSuffix(' mm')
        
        translate_y_label =QtWidgets.QLabel("Y")
        self.translate_y = QtWidgets.QDoubleSpinBox()
        self.translate_y.setMinimum(-1000)
        self.translate_y.setValue(0)
        self.translate_y.setMaximum(1000)
        self.translate_y.setPrefix('Y ')
        self.translate_y.setSuffix(' mm')
        
        translate_z_label =QtWidgets.QLabel("Z")
        self.translate_z = QtWidgets.QDoubleSpinBox()
        self.translate_z.setMinimum(-1000)
        self.translate_z.setValue(0)
        self.translate_z.setMaximum(1000)
        self.translate_z.setPrefix('Z ')
        self.translate_z.setSuffix(' mm')

        #make button group for STL origin rotation
        self.rotate_x = QtWidgets.QDoubleSpinBox()
        self.rotate_x.setSingleStep(15)
        self.rotate_x.setMinimum(-345)
        self.rotate_x.setValue(0)
        self.rotate_x.setMaximum(345)
        self.rotate_x.setPrefix('X ')
        self.rotate_x.setSuffix(' °')
        
        self.rotate_y = QtWidgets.QDoubleSpinBox()
        self.rotate_y.setSingleStep(15)
        self.rotate_y.setMinimum(-345)
        self.rotate_y.setValue(0)
        self.rotate_y.setMaximum(345)
        self.rotate_y.setPrefix('Y ')
        self.rotate_y.setSuffix(' °')
        
        self.rotate_z = QtWidgets.QDoubleSpinBox()
        self.rotate_z.setSingleStep(15)
        self.rotate_z.setMinimum(-345)
        self.rotate_z.setValue(0)
        self.rotate_z.setMaximum(345)
        self.rotate_z.setPrefix('Z ')
        self.rotate_z.setSuffix(' °')

        #scaling
        self.scale_x = QtWidgets.QDoubleSpinBox()
        self.scale_x.setSingleStep(1)
        self.scale_x.setMinimum(0.0001)
        self.scale_x.setValue(1)
        self.scale_x.setMaximum(10000)
        self.scale_x.setPrefix('X ')
        self.scale_x.setSuffix(' ×')
        
        self.scale_y = QtWidgets.QDoubleSpinBox()
        self.scale_y.setSingleStep(1)
        self.scale_y.setMinimum(0.0001)
        self.scale_y.setValue(1)
        self.scale_y.setMaximum(10000)
        self.scale_y.setPrefix('Y ')
        self.scale_y.setSuffix(' ×')
        
        self.scale_z = QtWidgets.QDoubleSpinBox()
        self.scale_z.setSingleStep(1)
        self.scale_z.setMinimum(0.0001)
        self.scale_z.setValue(1)
        self.scale_z.setMaximum(10000)
        self.scale_z.setPrefix('Z ')
        self.scale_z.setSuffix(' ×')
        
        
        self.scale_uniform_cb = QtWidgets.QCheckBox('Uniform scaling')

        #transform origin button layout
        translate_layout.addWidget(self.translate_x)
        translate_layout.addWidget(self.translate_y)
        translate_layout.addWidget(self.translate_z)
        
        rotate_layout.addWidget(self.rotate_x)
        rotate_layout.addWidget(self.rotate_y)
        rotate_layout.addWidget(self.rotate_z)
        
        scale_layout.addWidget(self.scale_x)
        scale_layout.addWidget(self.scale_y)
        scale_layout.addWidget(self.scale_z)
        
        translate_box.setLayout(translate_layout)
        rotate_box.setLayout(rotate_layout)
        scale_box.setLayout(scale_layout)
        if cond == 'translate':
            vl.addWidget(self.choose_vertex_button)
            vl.addWidget(self.first_centroid)
            hl.addWidget(translate_box)
        vl.addWidget(self.trans_origin_button)
        if cond == 'rotate':
            hl.addWidget(rotate_box)
        if cond == 'scale':
            vl.addWidget(self.scale_uniform_cb)
            hl.addWidget(scale_box)
        
        button_layout.addLayout(vl)
        button_layout.addLayout(hl)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    window = QtWidgets.QWidget()
    layout = make_transformation_button_layout(window)
    window.setLayout(layout)
    window.show()
    sys.exit(app.exec_())